@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -136,19 +136,19 @@ USA.
   (let ((modifier (slot-modifier <url> 'CONTAINER)))
     (lambda (url compute-container)
       (let ((string (url->string url)))
-	(or (hash-table/get interned-urls string #f)
+	(or (hash-table-ref/default interned-urls string #f)
 	    (begin
 	      (let ((finished? #f))
 		(dynamic-wind
 		 (lambda ()
-		   (hash-table/put! interned-urls string url))
+		   (hash-table-set! interned-urls string url))
 		 (lambda ()
 		   (modifier url (compute-container url))
 		   (set! finished? #t)
 		   unspecific)
 		 (lambda ()
 		   (if (not finished?)
-		       (hash-table/remove! interned-urls string)))))
+		       (hash-table-delete! interned-urls string)))))
 	      url))))))
 
 (define interned-urls
@@ -156,10 +156,10 @@ USA.
 
 (define (define-url-protocol name class)
   (define-method url-protocol ((url class)) url name)
-  (hash-table/put! url-protocols (string-downcase name) class))
+  (hash-table-set! url-protocols (string-downcase name) class))
 
 (define (url-protocol-name? name)
-  (hash-table/get url-protocols (string-downcase name) #f))
+  (hash-table-ref/default url-protocols (string-downcase name) #f))
 
 (define url-protocols
   (make-string-hash-table))
@@ -369,22 +369,23 @@ USA.
       (memoize-resource (constructor url))))
 
 (define (get-memoized-resource url #!optional error?)
-  (or (let ((resource (hash-table/get memoized-resources url #f)))
+  (or (let ((resource (hash-table-ref/default memoized-resources url #f)))
 	(and resource
 	     (let ((resource (weak-car resource)))
 	       ;; Delete memoization _only_ if URL-EXISTS?
 	       ;; unambiguously states non-existence.  An error is
 	       ;; often transitory.
-	       (if (and resource (ignore-errors (lambda () (url-exists? url))))
+	       (if (and (not (gc-reclaimed-object? resource))
+			(ignore-errors (lambda () (url-exists? url))))
 		   resource
 		   (begin
-		     (hash-table/remove! memoized-resources url)
+		     (hash-table-delete! memoized-resources url)
 		     #f)))))
       (and (if (default-object? error?) #f error?)
 	   (error "URL has no associated resource:" url))))
 
 (define (memoize-resource resource)
-  (hash-table/put! memoized-resources
+  (hash-table-set! memoized-resources
 		   (resource-locator resource)
 		   (weak-cons resource
 			      (lambda (resource)
@@ -392,21 +393,21 @@ USA.
   resource)
 
 (define (unmemoize-resource url)
-  (let ((r.c (hash-table/get memoized-resources url #f)))
+  (let ((r.c (hash-table-ref/default memoized-resources url #f)))
     (if r.c
 	(let ((resource (weak-car r.c)))
-	  (if resource
+	  (if (not (gc-reclaimed-object? resource))
 	      (begin
 		(let ((close (weak-cdr r.c)))
 		  (if close
 		      (close resource)))
-		(hash-table/remove! memoized-resources url)))))))
+		(hash-table-delete! memoized-resources url)))))))
 
 (define (%unmemoize-resource url)
-  (hash-table/remove! memoized-resources url))
+  (hash-table-delete! memoized-resources url))
 
 (define memoized-resources
-  (make-weak-eq-hash-table))
+  (make-key-weak-eq-hash-table))
 
 ;;;; Folder operations
 
@@ -751,7 +752,7 @@ USA.
 (define (reset-folder-order! order)
   (set-folder-order-tree! order #f)
   (let ((cache (folder-order-cache order)))
-    (if cache (hash-table/clear! cache))))
+    (if cache (hash-table-clear! cache))))
 
 (define (map-folder-index folder index)
   (let ((order (folder-order folder)))
@@ -783,14 +784,15 @@ USA.
                (< (cdr a) (cdr b))))))))
 
 (define make-integer-hash-table
-  (strong-hash-table/constructor int:remainder int:=))
+  (hash-table-constructor
+   (make-hash-table-type int:remainder int:= #f hash-table-entry-type:strong)))
 
 (define (%message-order-key message order index)
   (let ((compute-key
 	 (lambda () (cons ((folder-order-selector order) message) index)))
 	(cache (folder-order-cache order)))
     (if cache
-	(hash-table/intern! cache index compute-key)
+	(hash-table-intern! cache index compute-key)
 	(compute-key))))
 
 (define (index-order-key folder order index)
@@ -830,7 +832,7 @@ USA.
        (let ((compute-key
 	      (if cache
 		  (lambda (message index)
-		    (hash-table/intern! cache index
+		    (hash-table-intern! cache index
 		      (lambda () (cons (selector message) index))))
 		  (lambda (message index)
 		    (cons (selector message) index)))))
@@ -967,10 +969,9 @@ USA.
 		   (safe-accessors #t)
 		   (constructor #f)
 		   (print-procedure
-		    (standard-unparser-method 'HEADER-FIELD
-		      (lambda (header port)
-			(write-char #\space port)
-			(write (header-field-name header) port)))))
+		    (standard-print-method 'HEADER-FIELD
+		      (lambda (header)
+			(list (header-field-name header))))))
   (name #f read-only #t)
   (value #f read-only #t))
 
@@ -1029,12 +1030,12 @@ USA.
 (define (write-header-fields headers port)
   (encode-header-fields headers
     (lambda (string start end)
-      (write-substring string start end port))))
+      (write-string string port start end))))
 
 (define (write-header-field header port)
   (encode-header-field header
     (lambda (string start end)
-      (write-substring string start end port))))
+      (write-string string port start end))))
 
 (define (header-fields->string headers)
   (call-with-output-string
@@ -1051,7 +1052,7 @@ USA.
    (lambda (port)
      (encode-header-field-value value
        (lambda (string start end)
-	 (write-substring string start end port))))))
+	 (write-string string port start end))))))
 
 (define (get-first-header-field headers name error?)
   (let loop ((headers (->header-fields headers)))
@@ -1074,9 +1075,9 @@ USA.
 	  (else winner))))
 
 (define (get-all-header-fields headers name)
-  (list-transform-positive (->header-fields headers)
-    (lambda (header)
-      (string-ci=? name (header-field-name header)))))
+  (filter (lambda (header)
+	    (string-ci=? name (header-field-name header)))
+	  (->header-fields headers)))
 
 (define (get-first-header-field-value headers name error?)
   (let ((header (get-first-header-field headers name error?)))
