@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -223,16 +223,7 @@ finite_flonum_p (SCHEME_OBJECT x)
 bool
 flonum_is_finite_p (SCHEME_OBJECT x)
 {
-  return double_is_finite_p (FLONUM_TO_DOUBLE (x));
-}
-
-bool
-double_is_finite_p (double x)
-{
-  return
-    (((x > 1.0) || (x < -1.0))
-     ? (x != (x / 2.0))
-     : ((x <= 1.0) && (x >= -1.0)));
+  return (isfinite (FLONUM_TO_DOUBLE (x)));
 }
 
 bool
@@ -675,7 +666,7 @@ integer_shift_left (SCHEME_OBJECT n, unsigned long m)
       long n1 = (FIXNUM_TO_LONG (n));
       if (n1 < 0)
 	{
-	  if ((m + (ulong_length_in_bits (~n1))) < FIXNUM_LENGTH)
+	  if ((m + (ulong_length_in_bits (~n1))) <= FIXNUM_LENGTH)
 	    /* The behaviour of shifting a negative integer is
 	       undefined in C.  */
 	    return (LONG_TO_FIXNUM (- ((-n1) << m)));
@@ -685,7 +676,7 @@ integer_shift_left (SCHEME_OBJECT n, unsigned long m)
 	}
       else if (0 < n1)
 	{
-	  if ((m + (ulong_length_in_bits (n1))) < FIXNUM_LENGTH)
+	  if ((m + (ulong_length_in_bits (n1))) <= FIXNUM_LENGTH)
 	    return (LONG_TO_FIXNUM (n1 << m));
 	  else
 	    return (unsigned_long_to_shifted_bignum (n1, m, 0));
@@ -705,6 +696,8 @@ integer_shift_right (SCHEME_OBJECT n, unsigned long m)
   if (FIXNUM_P (n))
     {
       long n1 = (FIXNUM_TO_LONG (n));
+      if (m >= (CHAR_BIT * (sizeof (n1))))
+	return (LONG_TO_FIXNUM ((n1 < 0) ? (-1) : 0));
       return (LONG_TO_FIXNUM ((n1 < 0) ? (~ ((~n1) >> m)) : (n1 >> m)));
     }
   else
