@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -43,10 +43,14 @@ extern void outf_error (const char *, ...)
 extern void outf_fatal (const char *, ...)
   ATTRIBUTE ((__format__ (__printf__, 1, 2)));
 
+extern void outf_error_line (const char *, ...)
+  ATTRIBUTE ((__format__ (__printf__, 1, 2)));
+
 extern void voutf (outf_channel, const char *, va_list);
 extern void voutf_console (const char *, va_list);
 extern void voutf_error (const char *, va_list);
 extern void voutf_fatal (const char *, va_list);
+extern void voutf_error_line (const char *, va_list);
 
 extern void outf_flush (outf_channel chan);
 extern void outf_flush_console (void);
