@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -39,8 +39,15 @@ USA.
   (or (eq? object #f)
       (eq? object #t)))
 
-(define (boolean=? x y)
-  (if x y (not y)))
+;; R7RS says that the args to this procedure must satisfy boolean?.
+;; We relax that to allow any object.
+(define (boolean=? b1 b2 . bs)
+  (and (if b1 b2 (not b2))
+       (every (if b1 (lambda (b) b) not)
+	      bs)))
+
+(define (boolean<? b1 b2)
+  (and (not b1) b2))
 
 (define (boolean/or . arguments)
   (let loop ((arguments arguments))
@@ -56,26 +63,4 @@ USA.
 	(if (car arguments)
 	    (loop (cdr arguments))
 	    #f)
-	#t)))
-
-(define (there-exists? items predicate)
-  (let loop ((items* items))
-    (if (pair? items*)
-	(if (predicate (car items*))
-	    #t
-	    (loop (cdr items*)))
-	(begin
-	  (if (not (null? items*))
-	      (error:not-list items 'THERE-EXISTS?))
-	  #f))))
-
-(define (for-all? items predicate)
-  (let loop ((items* items))
-    (if (pair? items*)
-	(if (predicate (car items*))
-	    (loop (cdr items*))
-	    #f)
-	(begin
-	  (if (not (null? items*))
-	      (error:not-list items 'FOR-ALL?))
-	  #t))))
\ No newline at end of file
+	#t)))
\ No newline at end of file
