@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -140,7 +140,9 @@ USA.
     ("record"	(runtime record))
     ("reference-trap" (runtime reference-trap))
     ("regexp"	(runtime regular-expression))
-    ("regsexp"	(runtime regular-sexpression))
+    ("regexp-recursive"	(runtime regexp recursive))
+    ("regexp-rules"	(runtime regexp rules))
+    ("regsexp"	(runtime regexp regsexp))
     ("rep"	(runtime rep))
     ("rexp"	(runtime rexp))
     ("rfc2822-headers" (runtime rfc2822-headers))
@@ -152,6 +154,7 @@ USA.
     ("sfile"	(runtime simple-file-ops))
     ("socket"	(runtime socket))
     ("srfi-1"	(runtime srfi-1))
+    ("srfi-115"	(runtime regexp srfi-115))
     ("stack-sample" (runtime stack-sampler))
     ("stream"	(runtime stream))
     ("string"	(runtime string))
